/*
 * Named_os2.c by Antonio Sanmarti <warper@ansaro.com>.
 */

#ifdef __EMX__
#include "named_os2.h"
#include "port_before.h"

#ifdef HAVE_SPAWNXFER

#define WAIT_EXITED		0
#define STILL_ACTIVE		0x001

#ifndef BOOL
#define BOOL unsigned int
#endif

#ifndef TRUE
#define TRUE 1
#endif

#ifndef FALSE
#define FALSE 0
#endif

#ifndef ULONG
#define ULONG unsigned long
#endif

#include <process.h>

#include <sys/wait.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <arpa/nameser.h>
#include <errno.h>
#include <signal.h>
#include <resolv.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <syslog.h>
#include <time.h>
#include <unistd.h>
#include <isc/eventlib.h>
#include <isc/logging.h>
#include <isc/memcluster.h>
#include <isc/dst.h>
#include <isc/list.h>
#include <res_update.h>
#include "../../../bin/named/ns_defs.h"
#include "../../../bin/named/ns_glob.h"
#include "port_after.h"


void reapchildThread(void *);

/* Inicializa la lista de xfers */
static os2_xfer_init = 0;

/* Keep track of the status of spawned xfers */
static struct _xferstatus {
	pid_t pid;
	unsigned status;
} xfers[MAX_XFERS_RUNNING];

/*
 * Funcion principal encargada de llamar a named-xfer.exe
 */
pid_t spawnxfer(char **argv, struct zoneinfo *zp) {
	static pid_t pid, pid_fake;
	struct timeval t;
	int i,j,estado;
	int stacksize,ThreadID;
	void *pargs;

	/* Limpiamos la lista de pids */
	if (!os2_xfer_init) {
		os2_xfer_init=1;
		for(i = 0; i < MAX_XFERS_RUNNING;i++) {
			xfers[i].pid=(pid_t)0;
			xfers[i].status=0;
			}
		}

	/* pid = 0; */
	pid=getpid();

	sleep(1);
	/* Unica solucion viable para llamar a named-xfer.exe :-( */
	i=spawnv(P_SESSION | P_UNRELATED | P_MINIMIZE,server_options->named_xfer, argv);
	if(!i) estado = XFER_SUCCESSAXFR; else estado = XFER_FAIL;

	/* Creacion de un proceso falso. Si lo hacemos via fork(), los pids devueltos
	siempre son los mismos :-? */
	gettimeofday(&t,NULL);
	pid_fake=(pid_t)t.tv_sec;

	for(j = 0; j < MAX_XFERS_RUNNING;j++) {
		if (!xfers[j].pid) {
			xfers[j].pid=pid_fake;
			xfers[j].status=estado;
			stacksize= 32768;
			pargs=(void*)&pid;
			ThreadID = _beginthread(reapchildThread, NULL, stacksize, pargs);
			break;
			}
		}

	return(pid_fake);
}

/*
 * Funcion encargada de enviar la senyal SIGCHLD.
 * reapchild() (ns_maint.c) es la funcion a llamar cuando esta senyal es enviada.
 */
void reapchildThread(void *ppid) {
	int i,*pid;

	sleep(2); /* Damos tiempo a que se actualice la tabla xferstatus[] */
	pid=(int*)ppid;
	i=kill(*pid,SIGCHLD);
	_endthread();
}

/*
 * Called by reapchild() int ns_maint.c
 * emulate UNIX waitpid() for named. (sacado del port para el NT)
 */
int os2waitpid(pid_t pid, int *stat_loc, int options)
{
	int i;
	pid_t rc = 0;
	BOOL finished = FALSE;
	
	do {
	/* only check once if WNOHANG */
	if (options == WNOHANG) finished = TRUE;

	for(i = 0; i < MAX_XFERS_RUNNING; i++) {
		/* -1 indicates any child process, nonzero is a specific process */
		if(pid == -1 || (ULONG)pid == xfers[i].pid) {
			/* Check the state of the process */
			/* if(xfers[i].pid && (xfers[i].status != STILL_ACTIVE)) {*/
			if(xfers[i].pid) {
				finished = TRUE;
				if(stat_loc) {
					/* Put the exit code in the high byte of the high word */
					*stat_loc = xfers[i].status;
					*stat_loc <<= 8;
					/* Set the status */
					*stat_loc |= WAIT_EXITED;
					}
				/* Clear this process from the record */
				rc = (pid_t)xfers[i].pid;
				xfers[i].pid = (pid_t)0;
				xfers[i].status=0;
				ns_debug(ns_log_default, 15, "os2waitpid() returns %d", rc);
				/* We want to return the pid of this process */
				return(rc);
				}
			}
		}
	} while(!finished);
	return(rc);
}

#endif /* !HAVE_SPAWNXFER */
#endif /* !__EMX__*/







